/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of Stephane Sudre nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationWindowView.h"
#include <Carbon/Carbon.h>

#define ICPRESENTATIONWINDOWVIEW_SHADOW_OFFSET		10.0f
#define ICPRESENTATIONWINDOWVIEW_TITLE_BAR_HEIGHT	24.0f

@implementation ICPresentationWindowView

- (id) initWithFrame:(NSRect) inFrame
{
	self=[super initWithFrame:inFrame];
	
	if (self!=nil)
	{
	}
	
	return self;
}

- (void) dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	[super dealloc];
}

#pragma mark -

- (BOOL) isOpaque
{
	return NO;
}

- (void) drawRect:(NSRect) inFrame
{
	HIThemeWindowDrawInfo tWindowDrawInfo;
	HIRect tContentRect;
	HIRect tTitleRect;
	CGContextRef tContextRef;
	NSRect tBounds,tShadowFrame;
	NSShadow * tShadow;
	NSBezierPath * tBezierPath;
	
	tShadowFrame=tBounds=[self bounds];
	
	// Draw Shadow
	
	tShadowFrame.origin.x=ICPRESENTATIONWINDOWVIEW_SHADOW_OFFSET;
	tShadowFrame.origin.y=ICPRESENTATIONWINDOWVIEW_SHADOW_OFFSET;
	tShadowFrame.size.width-=2*ICPRESENTATIONWINDOWVIEW_SHADOW_OFFSET;
	tShadowFrame.size.height-=2*ICPRESENTATIONWINDOWVIEW_SHADOW_OFFSET;
	
	[NSGraphicsContext saveGraphicsState];
	
	tShadow=[[NSShadow alloc] init];
	
	[tShadow setShadowOffset:NSMakeSize(0,-3)];
	
	[tShadow setShadowBlurRadius:12.0f];
	
	if ([[self window] isMainWindow]==YES)
	{
		[tShadow setShadowColor:[[NSColor blackColor] colorWithAlphaComponent:0.5f]]; 
	}
	else
	{
		[tShadow setShadowColor:[[NSColor lightGrayColor] colorWithAlphaComponent:0.5f]]; 
	}
	
	[tShadow set];
	
	[[NSColor blackColor] set];
	
	NSRectFillUsingOperation(tShadowFrame,NSCompositeSourceOver);
	
	[NSGraphicsContext restoreGraphicsState];
	
	[tShadow release];
	
	// Draw Frame
	
	tContentRect.origin.x=ICPRESENTATIONWINDOWVIEW_SHADOW_OFFSET;
	tContentRect.origin.y=ICPRESENTATIONWINDOWVIEW_SHADOW_OFFSET;//;
	tContentRect.size.width=NSWidth(tBounds)-2*ICPRESENTATIONWINDOWVIEW_SHADOW_OFFSET;
	tContentRect.size.height=NSHeight(tBounds)-ICPRESENTATIONWINDOWVIEW_TITLE_BAR_HEIGHT-ICPRESENTATIONWINDOWVIEW_SHADOW_OFFSET;
	
	tWindowDrawInfo.version=0;
	
	if ([[self window] isMainWindow]==YES)
	{
		tWindowDrawInfo.state=kThemeStateActive;
	}
	else
	{
		tWindowDrawInfo.state=kThemeStateInactive;
	}
	
	tWindowDrawInfo.windowType=kThemeDocumentWindow;
	
	tWindowDrawInfo.attributes=kThemeWindowHasTitleText;
	
	tWindowDrawInfo.titleHeight=10;
	
	tWindowDrawInfo.titleWidth=100;
	
	tContextRef=(CGContextRef) [[[self window] graphicsContext] graphicsPort];

	
	
	HIThemeDrawWindowFrame(&tContentRect,&tWindowDrawInfo,tContextRef,kHIThemeOrientationInverted,&tTitleRect);
	
	tBounds=NSMakeRect(tContentRect.origin.x,tContentRect.origin.y,tContentRect.size.width,tContentRect.size.height);


	// Draw Frame
	
	[[NSColor colorWithDeviceWhite:0.7373f alpha:0.5f] set];
		
	tBezierPath=[NSBezierPath bezierPath];
	
	[tBezierPath moveToPoint:NSMakePoint(NSMinX(tBounds)-0.5f,NSMinY(tBounds)-0.5f)];
	
	[tBezierPath lineToPoint:NSMakePoint(NSMaxX(tBounds)+0.5f,NSMinY(tBounds)-0.5f)];
	
	[tBezierPath lineToPoint:NSMakePoint(NSMaxX(tBounds)+0.5f,NSMaxY(tBounds)+16.5f)];
	
	[tBezierPath appendBezierPathWithArcWithCenter:NSMakePoint(NSMaxX(tBounds)+0.5f-5.0f,NSMaxY(tBounds)+16.5f)
											radius:5.0f
										startAngle:0.0f
										  endAngle:90.0f];
	
	[tBezierPath lineToPoint:NSMakePoint(NSMinX(tBounds)+5.0f,NSMaxY(tBounds)+21.5f)];
	
	[tBezierPath appendBezierPathWithArcWithCenter:NSMakePoint(NSMinX(tBounds)-0.5f+5.0f,NSMaxY(tBounds)+16.5f)
											radius:5.0f
										startAngle:90.0f
										  endAngle:180.0f];
	
	[tBezierPath closePath];
	
	[tBezierPath stroke];
	
	// Draw Background

#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5
	
	tBounds=NSMakeRect(tContentRect.origin.x,tContentRect.origin.y-1.0f,tContentRect.size.width,tContentRect.size.height+1.0f);
	
#else
	
	tBounds=NSMakeRect(tContentRect.origin.x,tContentRect.origin.y,tContentRect.size.width,tContentRect.size.height);
	
#endif
	
	if (IBhelperWindow_!=nil)
	{
		[[IBhelperWindow_ backgroundColor] set];
	}
	else
	{
		[[[self window] backgroundColor] set];
	}
	
	NSRectFillUsingOperation(tBounds,NSCompositeSourceOver);
}

#pragma mark -

- (void) viewWillMoveToWindow:(NSWindow *) inWindow
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	if (inWindow!=nil)
	{
		// Register for Notification
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowDidBecomeMain:)
													 name:NSWindowDidBecomeMainNotification
												   object:inWindow];
												   
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowDidResignMain:)
													 name:NSWindowDidResignMainNotification
												   object:inWindow];
	}
}

- (void) windowDidBecomeMain:(NSNotification *) inNotification
{
	[self setNeedsDisplay:YES];
}

- (void) windowDidResignMain:(NSNotification *) inNotification
{
	[self setNeedsDisplay:YES];
}

@end
