#import "MainController.h"

NSString * const ACTION_SELECTOR_NAME=@"ACTION_SELECTOR_NAME";

NSString * const ACTION_DELAY=@"ACTION_DELAY";

#define SUB_WINDOW_HEIGHT	64.0
#define SUB_WINDOW_WIDTH	64.0


/*NSWindow * tWindow;
	
tTransparentWindow=[[NSWindow alloc] initWithContentRect:tContentFrame
											   styleMask:NSTitledWindowMask 
												 backing:NSBackingStoreBuffered 
												   defer:YES];*/





@interface ImageFramerView: NSView
{
	NSImage * image_;
	
	NSUInteger xIndex_;
	NSUInteger yIndex_;
}

- (void) setImage:(NSImage *) inImage;

- (void) setXIndex:(NSUInteger) inXIndex;

- (void) setYIndex:(NSUInteger) inYIndex;

@end

@implementation ImageFramerView

- (void) dealloc
{
	[image_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) setImage:(NSImage *) inImage
{
	image_=[inImage retain];
}

- (void) setXIndex:(NSUInteger) inXIndex
{
	xIndex_=inXIndex;
	
}

- (void) setYIndex:(NSUInteger) inYIndex
{
	yIndex_=inYIndex;
}

- (BOOL) isOpaque
{
	return YES;
}

- (BOOL) isFlipped
{
	return NO;
}

- (void) drawRect:(NSRect) inFrame
{
	NSRect tBounds;
	
	tBounds=[self bounds];
	
	[image_ drawInRect:NSMakeRect(0.0,0.0,NSWidth(tBounds),NSHeight(tBounds)) fromRect:NSMakeRect(xIndex_*SUB_WINDOW_WIDTH,yIndex_*SUB_WINDOW_HEIGHT,NSWidth(tBounds),NSHeight(tBounds))  operation:NSCompositeCopy fraction:1.0f];
}

@end


@implementation MainController

- (void) awakeFromNib
{
	contentView_=[IBwindow_ contentView];

	_closeButton=[IBwindow_ standardWindowButton:NSWindowCloseButton];
	
	_miniaturizeButton=[IBwindow_ standardWindowButton:NSWindowMiniaturizeButton];
	
	_zoomButton=[IBwindow_ standardWindowButton:NSWindowZoomButton];
	
	actions_=[NSArray arrayWithObjects:[NSDictionary dictionaryWithObjectsAndKeys:@"animationBackgroundColor:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:2.0f],ACTION_DELAY,nil],
									   [NSDictionary dictionaryWithObjectsAndKeys:@"animationDrawerRight:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:1.0f],ACTION_DELAY,nil],
									   [NSDictionary dictionaryWithObjectsAndKeys:@"animationDrawerLeft:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:0.0f],ACTION_DELAY,nil],
									   [NSDictionary dictionaryWithObjectsAndKeys:@"animationMinimize:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:1.0f],ACTION_DELAY,nil],
									   [NSDictionary dictionaryWithObjectsAndKeys:@"animationResize:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:1.0f],ACTION_DELAY,nil],
									   [NSDictionary dictionaryWithObjectsAndKeys:@"animationShake:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:1.0f],ACTION_DELAY,nil],
									   
									   [NSDictionary dictionaryWithObjectsAndKeys:@"animationAlertSheet:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:1.0f],ACTION_DELAY,nil],
									   [NSDictionary dictionaryWithObjectsAndKeys:@"animationButtons:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:0.5f],ACTION_DELAY,nil],
									   
									   
									   [NSDictionary dictionaryWithObjectsAndKeys:@"animationBrushedMetal:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:0.5f],ACTION_DELAY,nil],
			  
									  [NSDictionary dictionaryWithObjectsAndKeys:@"animationAlpha:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:1.0f],ACTION_DELAY,nil],
									  [NSDictionary dictionaryWithObjectsAndKeys:@"animationCustomWindow:",ACTION_SELECTOR_NAME,[NSNumber numberWithFloat:0.0f],ACTION_DELAY,nil],
									   nil];

	[actions_ retain];
}

#pragma mark -

- (void) animationAlertSheet:(id) inObject
{
	[NSApp beginSheet:IBsheet_ modalForWindow:IBwindow_ modalDelegate:self didEndSelector:nil contextInfo:NULL];
	
	[self performSelector:@selector(endAnimationAlertSheet:) withObject:nil afterDelay:1.0f];
}

- (void) endAnimationAlertSheet:(id) inObject
{
	[NSApp endSheet:IBsheet_];
	
	[IBsheet_ orderOut:self];
	
	[self runNextAction];
}

#pragma mark -

- (void) animationDrawerRight:(id) inObject
{
	[IBdrawer_ openOnEdge:NSMaxXEdge];
	
	[self performSelector:@selector(endAnimationDrawer:) withObject:nil afterDelay:1.0f];
}

- (void) animationDrawerLeft:(id) inObject
{
	[IBdrawer_ openOnEdge:NSMinXEdge];
	
	[self performSelector:@selector(endAnimationDrawer:) withObject:nil afterDelay:1.0f];
}

- (void) animationDrawerBottom:(id) inObject
{
	[IBdrawer_ openOnEdge:NSMinYEdge];
	
	[self performSelector:@selector(endAnimationDrawer:) withObject:nil afterDelay:1.0f];
}

- (void) endAnimationDrawer:(id) inObject
{
	[IBdrawer_ close];
	
	[self runNextAction];
}

#pragma mark -

- (void) animationMinimize:(id) inObject
{
	[IBwindow_ miniaturize:self];
	
	[self performSelector:@selector(endAnimationMinimize:) withObject:nil afterDelay:1.0f];
}

- (void) endAnimationMinimize:(id) inObject
{
	[IBwindow_ deminiaturize:self];
	
	[self runNextAction];
}

#pragma mark -

- (void) animationResize:(id) inObject
{
	NSRect tFrame;
	
	tFrame=[IBwindow_ frame];
	
	tFrame.origin.x-=NSWidth(tFrame)*0.5f;
	
	tFrame.size.width*=2.0f;
	
	[IBwindow_ setFrame:tFrame display:YES animate:YES];
	
	[self performSelector:@selector(endAnimationResize:) withObject:nil afterDelay:1.0f];
}

- (void) endAnimationResize:(id) inObject
{
	NSRect tFrame;
	
	tFrame=[IBwindow_ frame];
	
	tFrame.origin.x+=NSWidth(tFrame)*0.25f;
	
	tFrame.size.width*=0.5f;
	
	[IBwindow_ setFrame:tFrame display:YES animate:YES];
	
	[self runNextAction];
}

#pragma mark -

- (void) animationShake:(id) inObject
{
	unsigned int i,tCount;
	float tOffsetArray[]={-15.0f,15.0f,
						  -15.0f,15.0f,
						  -15.0f,15.0f,
						  -15.0f,15.0f,
						  -15.0f,15.0f,
						  -15.0f,15.0f,
						  0.0f};
	NSRect tFrame;
	struct timespec tTimespec={0,45000000};
	
	tFrame=[IBwindow_ frame];
	
	tCount=sizeof(tOffsetArray)/sizeof(float);
	
	for(i=0;i<tCount;i++)
	{
		[IBwindow_ setFrameOrigin:NSMakePoint(NSMinX(tFrame)+tOffsetArray[i],NSMinY(tFrame))];
		
		nanosleep(&tTimespec,NULL);
	}
	
	[self runNextAction];
}

#pragma mark -

- (void) animationAlpha:(id) inObject
{
	struct timespec tTimespec={0,45000000};
	float tAlpha=1.0f;
	
	for(;tAlpha>0.0f;tAlpha-=0.05f)
	{
		[IBwindow_ setAlphaValue:tAlpha];
		
		nanosleep(&tTimespec,NULL);
	}
	
	[IBwindow_ setAlphaValue:0.0f];
	
	/*sleep(1);
	
	for(;tAlpha<1.0;tAlpha+=0.05f)
	{
		[IBwindow_ setAlphaValue:tAlpha];
		
		nanosleep(&tTimespec,NULL);
	}
	*/
	 
	[IBwindow_ setAlphaValue:1.0f];
	
	[self runNextAction];
}

#pragma mark -

- (void) animationButtons:(id) inObject
{
	NSViewAnimation * tViewAnimation;
	NSRect tCloseStartRect;
	NSRect tZoomStartRect;
	static BOOL sBOOL=NO;
	
	sBOOL=!sBOOL;
	
	tCloseStartRect=[_closeButton frame];
	
	tZoomStartRect=[_zoomButton frame];
	
	[IBwindow_ setShowsResizeIndicator:NO];
	
	tViewAnimation=[[NSViewAnimation alloc] initWithViewAnimations:[NSArray arrayWithObjects:[NSDictionary dictionaryWithObjectsAndKeys:_closeButton,NSViewAnimationTargetKey,
																																		[NSValue valueWithRect:tCloseStartRect],NSViewAnimationStartFrameKey,
																																		[NSValue valueWithRect:tZoomStartRect],NSViewAnimationEndFrameKey,
																																		nil],
																							 [NSDictionary dictionaryWithObjectsAndKeys:_miniaturizeButton,NSViewAnimationTargetKey,
																																		(sBOOL==YES) ? NSViewAnimationFadeOutEffect:NSViewAnimationFadeInEffect,NSViewAnimationEffectKey,
																																		nil],
																							 [NSDictionary dictionaryWithObjectsAndKeys:_zoomButton,NSViewAnimationTargetKey,
																																		[NSValue valueWithRect:tZoomStartRect],NSViewAnimationStartFrameKey,
																																		[NSValue valueWithRect:tCloseStartRect],NSViewAnimationEndFrameKey,
																																		nil],
																							 nil]];
	
	
	[tViewAnimation setDelegate:self];
	
	[tViewAnimation startAnimation];
}

- (void) animationDidEnd:(NSAnimation *) inAnimation
{
	static int sCount=0;
	
	[inAnimation release];
	
	sCount++;
	
	if (sCount==8)
	{
		sCount=0;
		
		[IBwindow_ setShowsResizeIndicator:YES];
	
		[self runNextAction];
	}
	else
	{
		NSButton * tButton;
		
		tButton=_closeButton;
		
		_closeButton=_zoomButton;
		
		_zoomButton=tButton;
		
		[self animationButtons:nil];
	}
}

#pragma mark -

- (void) animationBackgroundColor:(id) inObject
{
	NSView * tView;
	
	tView=[[NSView alloc] initWithFrame:NSZeroRect];
	
	[contentView_ retain];
	
	[IBwindow_ setContentView:tView];
	
	[IBwindow_ setBackgroundColor:[NSColor colorWithPatternImage:[NSImage imageNamed:@"Logo_CocoaHeads_Small"]]];

	[self performSelector:@selector(endAnimationBackgroundColor:) withObject:nil afterDelay:2.0f];
}

- (void) endAnimationBackgroundColor:(id) inObject
{
	[IBwindow_ setContentView:contentView_];
	
	[contentView_ release];
	
	[IBwindow_ setBackgroundColor:[NSColor windowBackgroundColor]];
	
	[self runNextAction];
}

#pragma mark -

- (void) animationBrushedMetal:(id) inObject
{
	[IBtextured_ makeKeyAndOrderFront:self];
	
	[IBwindow_ orderOut:self];
	
	[self performSelector:@selector(endAnimationBrushedMetal:) withObject:nil afterDelay:1.5f];
}

- (void) endAnimationBrushedMetal:(id) inObject
{
	[IBwindow_ makeKeyAndOrderFront:self];
	
	[IBtextured_ orderOut:self];
	
	[self runNextAction];
}

#pragma mark -

- (void) animationCustomWindow:(id) inObject
{
	CGImageRef tImageRef;
	NSRect tWindowFrame;
	CGFloat x,y;
	NSSize tSnapshotSize;
	NSUInteger i=0,j;
	NSImage * tImage;
	NSMutableArray * tAnimationArray;
	NSViewAnimation * tViewAnimation;
	NSBitmapImageRep * tBitmapRep;
	
	// Capture the window and its shadow (10.5 and later only)
	
	tImageRef = CGWindowListCreateImage(CGRectNull, kCGWindowListOptionIncludingWindow, [IBwindow_ windowNumber], kCGWindowImageDefault);
	
	tBitmapRep = [[NSBitmapImageRep alloc] initWithCGImage:tImageRef];
	
	tSnapshotSize=[tBitmapRep size];
	
	tImage = [[NSImage alloc] initWithSize:tSnapshotSize];
	
	[tImage addRepresentation:tBitmapRep];
	
	[tBitmapRep release];
	
	
	CFRelease(tImageRef);
	
	// Create a transparent window and reproduce the window from square of NSView
	
	tWindowFrame=[IBwindow_ frame];

	tWindowFrame.origin.y-=15;
	
	tAnimationArray=[NSMutableArray array];
	
	NSWindow * tTransparentWindow;
	
	tTransparentWindow=[[NSWindow alloc] initWithContentRect:NSZeroRect styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:YES];
	
	if (tTransparentWindow!=nil)
	{
		NSView * tContentView;
		
		NSRect tTransparentWindowFrame;
		
		tTransparentWindowFrame=[[NSScreen mainScreen] visibleFrame];
		
		tTransparentWindowFrame.origin.y-=4.0f;
		
		[tTransparentWindow setFrame:tTransparentWindowFrame display:NO];
		
		[tTransparentWindow setOpaque:NO];
		
		[tTransparentWindow setBackgroundColor:[NSColor clearColor]];
		
		tContentView=[tTransparentWindow contentView];
		
		for(x=floor(NSMidX(tWindowFrame)-tSnapshotSize.width*0.5);x<floor(NSMidX(tWindowFrame)+tSnapshotSize.width*0.5);x+=SUB_WINDOW_WIDTH,i++)
		{
			j=0;
			
			for(y=floor(NSMidY(tWindowFrame)-tSnapshotSize.height*0.5);y<floor(NSMidY(tWindowFrame)+tSnapshotSize.height*0.5);y+=SUB_WINDOW_HEIGHT,j++)
			{
				NSRect tNewFrame;
				ImageFramerView * tFramerView;
				NSRect tStartFrame;
				NSRect tEndFrame;
				CGFloat tFinalX;
					
				tNewFrame=NSMakeRect(x, y, SUB_WINDOW_WIDTH, SUB_WINDOW_HEIGHT);
				
				tFramerView=[[ImageFramerView alloc] initWithFrame:tNewFrame];
					
				[tFramerView setImage:tImage];
				
				[tFramerView setXIndex:i];
				
				[tFramerView setYIndex:j];
				
				[tContentView addSubview:tFramerView];
				
				[tFramerView release];
				
				tStartFrame=[tFramerView frame];
					
				tFinalX=(rand()*NSWidth(tTransparentWindowFrame))/RAND_MAX;
				
				if (tFinalX<NSMidX(tTransparentWindowFrame))
				{
					tFinalX-=NSWidth(tTransparentWindowFrame)*0.6f;
				}
				else
				{
					tFinalX+=NSWidth(tTransparentWindowFrame)*0.6f;
				}

				tEndFrame=NSMakeRect(tFinalX,NSMinY(tStartFrame), NSWidth(tStartFrame), NSHeight(tStartFrame));
				
				[tAnimationArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:tFramerView,NSViewAnimationTargetKey,
																					 [NSValue valueWithRect:tEndFrame],NSViewAnimationStartFrameKey,
																					 [NSValue valueWithRect:tStartFrame],NSViewAnimationEndFrameKey,
											nil]];
			}
			
		}
		
		[tTransparentWindow orderFront:self];
	}
	
	[IBwindow_ orderOut:self];
	
	tViewAnimation=[[NSViewAnimation alloc] initWithViewAnimations:tAnimationArray];
	
	[tViewAnimation setAnimationCurve:NSAnimationLinear];
	
	[tViewAnimation setDuration:2.0f];
	
	[tViewAnimation startAnimation];
}

#pragma mark -

- (void) runNextAction
{
	NSDictionary * tActionDictionary;
	
	if (actionIndex_<[actions_ count])
	{
		NSString * tActionName;
		NSNumber * tActionDelay;
		
		tActionDictionary=[actions_ objectAtIndex:actionIndex_];
		
		tActionName=[tActionDictionary objectForKey:ACTION_SELECTOR_NAME];
		
		tActionDelay=[tActionDictionary objectForKey:ACTION_DELAY];
		
		actionIndex_++;
		
		if (tActionName!=nil && tActionDelay!=nil)
		{
			[self performSelector:NSSelectorFromString(tActionName) withObject:nil afterDelay:[tActionDelay floatValue]];
		}
		else
		{
			[self runNextAction];
		}
	}
	else
	{
		animating_=NO;

		actionIndex_=0;
	}
}

- (IBAction) startAnimation:(id) sender
{
	if (animating_==NO)
	{
		animating_=YES;
	
		actionIndex_=0;
		
		[self runNextAction];
	}
}

- (IBAction) stopAnimation:(id) sender
{
	animating_=NO;

	actionIndex_=0;
	
	[NSObject cancelPreviousPerformRequestsWithTarget:self];
	
}

@end
